require "wechat_public/version"
require "wechat/api"
module WechatPublic
  autoload :Message, "wechat/message"
  autoload :Responder, "wechat/responder"
  autoload :Response, "wechat/response"
  autoload :Test, 'wechat/test'
  API_BASE = "https://api.weixin.qq.com/cgi-bin/"
  TICKET_URL = "https://api.weixin.qq.com/cgi-bin/qrcode/create?access_token=";
  #WechatPublic::Test.print
  class AccessTokenExpiredError < StandardError; end
  class ResponseError < StandardError
    attr_reader :error_code
    def initialize(errcode, errmsg)
      error_code = errcode
      super "#{errmsg}(#{error_code})"
    end
  end

  attr_reader :config

  def self.config 
    # @config ||= begin
    #   if defined? Rails
    #     config_file = Rails.root.join("config/wechat.yml")
    #     config = YAML.load(ERB.new(File.new(config_file).read).result)[Rails.env] if (File.exist?(config_file))
    #   end

    #   config ||= {appid: ENV["WECHAT_APPID"], secret: ENV["WECHAT_SECRET"], token: ENV["WECHAT_TOKEN"], access_token: ENV["WECHAT_ACCESS_TOKEN"]}
    #   config.symbolize_keys!
    #   config[:access_token] ||= Rails.root.join("tmp/access_token").to_s
    #   puts '======'
    #   puts config
    #   OpenStruct.new(config)
    # end
    @config ||= begin
      config = {};
      yield config
      puts '======',config
      @config = OpenStruct.new(config)        
    end
  end


  def self.api
    @api ||= WechatPublic::Api.new(self.config.appid, self.config.secret, self.config.access_token)
  end

  def self.get_token
    unless  @access_token 
      client = WechatPublic::Client.new(API_BASE)
      @access_token = WechatPublic::AccessToken.new(client, self.config.appid, self.config.secret, self.config.access_token)      
    end
    @access_token.token_data 
  end

  def self.token
    unless @access_token 
      client = WechatPublic::Client.new(API_BASE)
      @access_token = WechatPublic::AccessToken.new(client, self.config.appid, self.config.secret, self.config.access_token)      
    end    
    @access_token.token 
  end

  def self.generate_ticket hash
    hash ||= {}
    expire_seconds = hash.fetch("expire_seconds",10);
    action_name = hash.fetch("action_name","QR_LIMIT_SCENE")
    scene_id = hash.fetch("scene_id",Random.new.rand(100..10000))
    form = {
      expire_seconds: expire_seconds,
      action_name: action_name,
      action_info:{
        scene:{
          scene_id: scene_id
        }
      }
    }
    body = RestClient.post(TICKET_URL+self.token,form.to_json,json_header)
    body = JSON.parse(body)
    puts body
    body
  end
  private
  def self.json_header
    { content_type: :json, accept: :json }
  end
end

puts "before 判断 ActionController::Base...加载了么",ActionController::Base
if defined? ActionController::Base
  class ActionController::Base
    def self.wechat_responder opts={}
      self.send(:include, WechatPublic::Responder)
      if (opts.empty?)
        self.wechat = WechatPublic.api
        self.token = WechatPublic.config.token
      else
        self.wechat = WechatPublic::Api.new(opts[:appid], opts[:secret], opts[:access_token])
        self.token = opts[:token]
      end
    end
  end
end

